﻿Public Class Form2

    Private Sub Form2_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        rtb_SnifferLog.Location = pbox_ScrollingScope.Location
        rtb_SnifferLog.Size = pbox_ScrollingScope.Size
        SetScaleLabels()
    End Sub

    Private Sub Form2_VisibleChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.VisibleChanged, _
                                                                                                    Me.SizeChanged
        StartTimer()
    End Sub

    Friend Sub StopTimer()
        Timer_100Hz.Stop()
    End Sub

    Friend Sub StartTimer()
        If Visible And WindowState <> FormWindowState.Minimized Then
            Timer_100Hz.Interval = 10
            Timer_100Hz.Start()
        Else
            Timer_100Hz.Stop()
        End If
    End Sub

    Private Sub Form2_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        GroupBox1.BackColor = Color.LightGoldenrodYellow
        TheSystem_StopAllSnifferLogs()
        Me.Hide()
        e.Cancel = True
    End Sub

    Private Sub Form2_LocationChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.LocationChanged
        If Not EventsAreEnabled Then Exit Sub
        LimitFormPosition(Me)
    End Sub


    Private Sub Timer_100Hz_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer_100Hz.Tick
        ShowPinDetails()
    End Sub


    Private AlternatePin As Pin
    Private SelectedPin As Pin

    Friend Sub DeselectAllPins()
        SelectedPin = Nothing
        AlternatePin = Nothing
        rtb_SnifferLog.Visible = False
        pbox_ScrollingScope.Visible = True
        TheSystem_StopAllSnifferLogs()
        LabelPin1.Text = "- - -"
        lbl_Details1.Text = ""
        FillPictureBox(pbox_Details1, 0, Color.Orange, Color.AliceBlue)
        LabelPin2.Text = "- - -"
        lbl_Details2.Text = ""
        FillPictureBox(pbox_Details2, 0, Color.Orange, Color.AliceBlue)
    End Sub


    Friend Sub SetPinDetails()
        Dim p As Pin = GetSelectedPin()
        '
        If p Is Nothing Then Return
        '
        If p Is SelectedPin Then
            AlternatePin = Nothing
            SelectedPin = p
        Else
            AlternatePin = SelectedPin
            SelectedPin = p
        End If
        '
        If SelectedPin IsNot Nothing Then
            LabelPin1.Text = ComposePinString(SelectedPin)
        End If
        If AlternatePin IsNot Nothing Then
            LabelPin2.Text = ComposePinString(AlternatePin)
        Else
            LabelPin2.Text = "- - -"
            lbl_Details2.Text = ""
            FillPictureBox(pbox_Details2, 0, Color.Orange, Color.AliceBlue)
        End If
        ' ---------------------------------------------------------------------- visualize sniffer log
        If SelectedPin IsNot Nothing AndAlso SelectedPin.GetPinType = Pin.PinTypes.SNIFFER AndAlso Me.Visible Then
            If Not SelectedPin.SnifferLogEnabled Then
                If btn_TextLog.Checked Then
                    rtb_SnifferLog.Visible = True
                    pbox_ScrollingScope.Visible = False
                    GroupBox1.BackColor = Color.Tomato
                    TheSystem_StopAllSnifferLogs()
                    rtb_SnifferLog.Text = ""
                    SelectedPin.SnifferLogEnabled = True
                End If
            Else
                If Not btn_TextLog.Checked Then
                    rtb_SnifferLog.Visible = False
                    pbox_ScrollingScope.Visible = True
                End If
                GroupBox1.BackColor = Color.LightGoldenrodYellow
                TheSystem_StopAllSnifferLogs()
            End If
        End If
        '
        TheSystem_CalibrateRawZero()
        '
        SetScaleLabels()
    End Sub

    Private Sub btn_GraphicLog_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_TextLog.ClickButtonArea
        If Not btn_TextLog.Checked Then
            rtb_SnifferLog.Visible = False
            pbox_ScrollingScope.Visible = True
            GroupBox1.BackColor = Color.LightGoldenrodYellow
            TheSystem_StopAllSnifferLogs()
        Else
            rtb_SnifferLog.Visible = True
            pbox_ScrollingScope.Visible = False
        End If
    End Sub

    Private Sub ShowPinDetails()
        ShowDetails()
        UpdateScrollingScope()
        BeepOnChange()
    End Sub

    Private Function ComposePinString(ByVal p As Pin) As String
        Return "M:" & (p.MasterId + 1).ToString & " S:" & (p.SlaveId + 1).ToString & " Pin:" & (p.PinId + 1).ToString
    End Function

    Private Sub ShowDetails()
        If SelectedPin IsNot Nothing Then
            lbl_Details1.Text = SelectedPin.GetValueString()
            FillPictureBox(pbox_Details1, SelectedPin.Value_Normalized, Color.Orange, Color.AliceBlue)
        End If
        If AlternatePin IsNot Nothing Then
            lbl_Details2.Text = AlternatePin.GetValueString()
            FillPictureBox(pbox_Details2, AlternatePin.Value_Normalized, Color.Orange, Color.AliceBlue)
        End If
    End Sub

    Private Sub BeepOnChange()
        If Not btn_BeepOnChange.Checked Then Return
        If SelectedPin IsNot Nothing Then
            Dim c As Double = SelectedPin.Value
            Static oldc As Double
            If c <> oldc And c <> 0 Then
                'Debug.Print(SelectedPin.Value.ToString)
                Beep()
                oldc = c
            End If
        End If
    End Sub

    Private Sub btn_SetRawZero_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_SetRawZero.ClickButtonArea
        TheSystem_CalibrateRawZero()
        SetScaleLabels()
    End Sub

    Private Sub btn_ShowRawCount_ClickButtonArea(ByVal Sender As System.Object, ByVal e As System.EventArgs) Handles btn_ShowRawCount.ClickButtonArea
        TheSystem_CalibrateRawZero()
        If btn_ShowRawCount.Checked Then
            tkbar_VerticalScaleZoom.Value = 100
        Else
            tkbar_VerticalScaleZoom.Value = 1
        End If
        SetScaleLabels()
    End Sub

    Private Sub UpdateScrollingScope()
        If Not pbox_ScrollingScope.Visible Then Return
        Dim v1 As Single = -999
        Dim v2 As Single = -999
        Dim zoom As Single = tkbar_VerticalScaleZoom.Value
        If btn_ShowRawCount.Checked Then
            If SelectedPin IsNot Nothing Then
                v1 = (SelectedPin.Value_RawInteger_Zero - CInt(SelectedPin.Value_RawInteger)) * zoom / 100.0F
            End If
            If AlternatePin IsNot Nothing Then
                v2 = (AlternatePin.Value_RawInteger_Zero - CInt(AlternatePin.Value_RawInteger)) * zoom / 100.0F
            End If
        Else
            If SelectedPin IsNot Nothing Then
                v1 = pbox_ScrollingScope.Height * 0.49F * (0.5F - SelectedPin.Value_Normalized) * zoom
            End If
            If AlternatePin IsNot Nothing Then
                v2 = pbox_ScrollingScope.Height * 0.49F * (0.5F - AlternatePin.Value_Normalized) * zoom
            End If
        End If
        DisplayCounterscope(pbox_ScrollingScope, v1, v2, tkbar_ScrollSpeed.Value)
    End Sub

    Private Sub SetScaleLabels()
        Label_Scale3.Text = ""
        Label_Scale2.Text = ""
        Label_Scale1.Text = ""
        Label_Scale3b.Text = ""
        Label_Scale2b.Text = ""
        Label_Scale1b.Text = ""
        Dim zoom As Double = tkbar_VerticalScaleZoom.Value
        If btn_ShowRawCount.Checked Then
            If SelectedPin IsNot Nothing Then
                Label_Scale3.Text = (SelectedPin.Value_RawInteger_Zero + 6800 / zoom).ToString("0")
                Label_Scale2.Text = SelectedPin.Value_RawInteger_Zero.ToString("0")
                Label_Scale1.Text = (SelectedPin.Value_RawInteger_Zero - 6800 / zoom).ToString("0")
            End If
            If AlternatePin IsNot Nothing Then
                Label_Scale3b.Text = (AlternatePin.Value_RawInteger_Zero + 6800 / zoom).ToString("0")
                Label_Scale2b.Text = AlternatePin.Value_RawInteger_Zero.ToString("0")
                Label_Scale1b.Text = (AlternatePin.Value_RawInteger_Zero - 6800 / zoom).ToString("0")
            End If
        Else
            Label_Scale3.Text = (500 + 500 / zoom).ToString("0")
            Label_Scale2.Text = "500"
            Label_Scale1.Text = (500 - 500 / zoom).ToString("0")
        End If
    End Sub

    Private Sub tkbar_VerticalScaleZoom_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tkbar_VerticalScaleZoom.Scroll
        SetScaleLabels()
    End Sub


End Class