﻿
Module ThereminoSystem

    Friend ConfigurationIsValid As Boolean = False
    Friend Masters As List(Of Master)

    Friend Sub TheSystem_InitMasters()
        If Masters IsNot Nothing Then
            'For Each m As Master In Masters
            '    '    If m.Hid.HidDevice_Unix.IsHidOpen() Then
            '    '        m.Hid.HidDevice_Unix.HidClose()
            '    '    End If
            '    'm.Hid = Nothing
            'Next
            Masters.Clear()
        Else
            Masters = New List(Of Master)
        End If

        If CanUseWindowsDlls Then
            Theremino_HID.FindThereminoHids()
        Else
            Theremino_HID.FindThereminoHids_Unix()
        End If

        ' -------------------------------------------- read Master names
        Dim i As Int32 = 0
        While i >= 0 And i < Masters.Count
            ' ---------------------------------------- read name two times to detect if the Master is already used
            Masters(i).ReadNameFromHardware()
            Dim name1 As String = Masters(i).GetName
            Masters(i).ReadNameFromHardware()
            Dim name2 As String = Masters(i).GetName
            ' ---------------------------------------- if a Master is used by another HAL then release it 
            If name1 <> name2 Then
                Masters.Remove(Masters(i))
                i -= 1
            End If
            i += 1
        End While
    End Sub

    'Private Function IncreaseFileNameIndex(ByVal filename As String) As String
    '    If filename = "" Then Return "Noname_01"
    '    '
    '    Dim path As String = IO.Path.GetDirectoryName(filename)
    '    Dim name As String = IO.Path.GetFileNameWithoutExtension(filename)
    '    Dim ext As String = IO.Path.GetExtension(filename)
    '    '
    '    If name = "" Then name = "NoName"
    '    ' ------------------------------------------------ find index chars
    '    Dim indexlen As Int32 = 0
    '    Dim i As Int32 = name.Length - 1
    '    While Char.IsNumber(name(i)) And i > 0
    '        i -= 1
    '        indexlen += 1
    '    End While
    '    ' ------------------------------------------------ increase index value
    '    If indexlen > 0 Then
    '        Dim number As Int32 = CInt(Val(Mid(name, i + 2)))
    '        number += 1
    '        name = Left(name, i + 1) & number.ToString(StrDup(indexlen, "0"))
    '    Else
    '        ' -------------------------------------------- or set index to "_01"
    '        name = name & "_01"
    '    End If
    '    ' ------------------------------------------------ compose final file-name
    '    Return PlatformAdjustedFileName(path & "\" & name & ext)
    'End Function

    Friend Sub TheSystem_StartTimers()
        If Not ConfigurationIsValid Then Exit Sub
        For Each m As Master In Masters
            m.StartTimedOperations()
        Next
    End Sub

    Friend Sub TheSystem_StopTimers()
        For Each m As Master In Masters
            m.StopTimedOperations()
        Next
    End Sub

    Friend Sub TheSystem_RecognizeSlaves()
        For Each m As Master In Masters
            m.Recognize()
        Next

        AssignDefaultSlotsToPins()
        AssignConfigurationsToMasters()
        SetConfigurationParamsToDevices()

        Form1.SelectedLine = -1
        TheSystem_ListComponents()
        Form1.ListView_SetAllLineColors()
        MarkErrorLines()
        Form1.Menu_File_Validate.Enabled = Not ConfigurationIsValid
        Form1.ToolStripButton_Validate.Enabled = Not ConfigurationIsValid
        Form1.EnableCalibrateButton(TheSystem_CalibrationNeeded)
        If IsSnifferPresent() Then
            Form2.btn_TextLog.Enabled = True
        Else
            Form2.btn_TextLog.Enabled = False
            Form2.btn_TextLog.Checked = False
        End If

        TheSystem_Calibrate()
    End Sub

    Private Sub AssignDefaultSlotsToPins()
        Dim i As Int32 = 1
        For Each m As Master In Masters
            Dim j As Int32 = i
            i += 100
            For Each s As Slave In m.Slaves
                For Each p As Pin In s.Pins
                    p.Slot = j
                    j += 1
                Next
            Next
        Next
    End Sub

    Private Function IsSnifferPresent() As Boolean
        For Each m As Master In Masters
            For Each s As Slave In m.Slaves
                For Each p As Pin In s.Pins
                    If p.GetPinType = Pin.PinTypes.SNIFFER Then
                        Return True
                    End If
                Next
            Next
        Next
        Return False
    End Function

    Friend Function TheSystem_CalibrationNeeded() As Boolean
        For Each m As Master In Masters
            If m.CalibrationNeeded Then Return True
        Next
    End Function

    Friend Sub TheSystem_Calibrate()
        For Each m As Master In Masters
            m.Calibrate()
        Next
    End Sub

    Friend Sub TheSystem_CalibrateRawZero()
        For Each m As Master In Masters
            m.CalibrateRawZero()
        Next
    End Sub

    Friend Sub TheSystem_SetParkingPositions()
        For Each m As Master In Masters
            m.SetOutputPinsToParkingPosition()
        Next
    End Sub

    Friend Function TheSystem_GetSlaveCount() As Int32
        Dim sc As Int32 = 0
        For Each m As Master In Masters
            sc += m.SlaveCount
        Next
        Return sc
    End Function

    Friend Sub TheSystem_StopAllSnifferLogs()
        For Each m As Master In Masters
            m.StopAllSnifferLogs()
        Next
    End Sub


    Friend Function GetMasterByListLine(ByVal line As Int32, ByVal LineType As String) As Master
        Dim m As Master = Nothing
        Dim s As Slave = Nothing
        Dim p As Pin = Nothing
        '
        Select Case LineType
            Case "Master"
                m = FindMasterByListLine(line)
            Case "Slave"
                s = FindSlaveByListLine(line)
                If s IsNot Nothing Then m = Masters(s.MasterId)
            Case "Pin"
                p = FindPinByListLine(line)
                If p IsNot Nothing Then m = Masters(p.MasterId)
        End Select
        '
        Return m
    End Function

    Private Function FindMasterByListLine(ByVal line As Int32) As Master
        Dim n As Int32 = 0
        For Each m As Master In Masters
            If n = line Then Return m
            n += 1
            For Each s As Slave In m.Slaves
                n += 1
                For Each p As Pin In s.Pins
                    n += 1
                Next
            Next
        Next
        Return Nothing
    End Function

    Private Function FindSlaveByListLine(ByVal line As Int32) As Slave
        Dim n As Int32 = 0
        For Each m As Master In Masters
            n += 1
            For Each s As Slave In m.Slaves
                If n = line Then Return s
                n += 1
                For Each p As Pin In s.Pins
                    n += 1
                Next
            Next
        Next
        Return Nothing
    End Function

    Friend Function FindPinByListLine(ByVal line As Int32) As Pin
        Dim n As Int32 = 0
        For Each m As Master In Masters
            n += 1
            For Each s As Slave In m.Slaves
                n += 1
                For Each p As Pin In s.Pins
                    If n = line Then Return p
                    n += 1
                Next
            Next
        Next
        Return Nothing
    End Function

    Friend Function GetSelectedPin() As Pin
        Return FindPinByListLine(Form1.SelectedLine)
    End Function



    ' ==================================================================================================
    '   LIST COMPONENTS
    ' ==================================================================================================

    Friend Sub TheSystem_ListComponents()
        Dim item As ListViewItem
        With Form1.MyListView1
            .GridLines = True
            .FullRowSelect = True
            .Header_BackColor1 = Color.FromArgb(255, 255, 200)
            .Header_BackColor2 = Color.FromArgb(200, 200, 200)
            .Header_ForeColor = Color.FromArgb(20, 20, 20)
            .Header_ShadowColor = Color.FromArgb(255, 255, 255)
            .Header_Font = New Font("Tahoma", 9, FontStyle.Regular)
            .Font = New Font("Courier new", 8)
            .View = View.Details
            .Clear()
            ' ----------------------------------------------------------------- column header
            .Columns.Add("Type", 56, HorizontalAlignment.Left)
            .Columns.Add("ID", 28, HorizontalAlignment.Left)
            .Columns.Add("Subtype", 105, HorizontalAlignment.Left)
            .Columns.Add("Dir.", 36, HorizontalAlignment.Left)
            .Columns.Add("Slot", 35, HorizontalAlignment.Center)
            .Columns.Add("Value", 62, HorizontalAlignment.Right)
            .Columns.Add("Notes", 200, HorizontalAlignment.Left)
            .Refresh()
            ' -----------------------------------------------------------------
            If CanUseWindowsDlls Then
                .SetExtStyle_DoubleBuffering()
                .BeginUpdate()
            End If
            For Each m As Master In Masters
                item = .Items.Add("Master")
                item.SubItems.Add((m.MasterId + 1).ToString)
                item.SubItems.Add(m.GetName)
                item.SubItems.Add("")
                item.SubItems.Add("")
                item.SubItems.Add("")
                item.SubItems.Add("")
                For Each s As Slave In m.Slaves
                    item = .Items.Add(" Slave")
                    item.SubItems.Add((s.SlaveId + 1).ToString)
                    item.SubItems.Add(s.SlaveTypeString)
                    item.SubItems.Add("")
                    item.SubItems.Add("")
                    item.SubItems.Add("")
                    item.SubItems.Add("")
                    For Each p As Pin In s.Pins
                        item = .Items.Add("   Pin")
                        item.SubItems.Add((p.PinId + 1).ToString)
                        item.SubItems.Add(Pin.PinTypeToString(p.GetPinType))
                        item.SubItems.Add(p.GetDirectionString)
                        If p.GetPinType <> Pin.PinTypes.UNUSED Then
                            item.SubItems.Add(p.Slot.ToString)
                            item.SubItems.Add("0.0")
                        Else
                            item.SubItems.Add("")
                            item.SubItems.Add("")
                        End If
                        item.SubItems.Add("")
                    Next
                Next
            Next
            TheSystem_ListViewResize()
            If CanUseWindowsDlls Then
                .EndUpdate()
            End If
        End With
    End Sub

    Private Sub MarkErrorLine(ByVal LineNumber As Int32)
        With Form1.MyListView1.Items(LineNumber)
            .SubItems(6).Text = "Found but not configured"
            .BackColor = Color.FromArgb(255, 150, 150)
            .ForeColor = Color.Black
        End With
    End Sub

    Private Sub AddErrorLine(ByVal config As String(), ByVal LineNumber As Int32)
        With Form1.MyListView1
            Dim item As ListViewItem
            Dim s() As String = Split(config(LineNumber), ",")
            item = .Items.Add(s(0))
            item.SubItems.Add("")
            item.SubItems.Add(s(1))
            item.SubItems.Add("")
            item.SubItems.Add("")
            item.SubItems.Add("")
            item.SubItems.Add("Configured but not found")
            item.BackColor = Color.FromArgb(255, 80, 50)
            item.ForeColor = Color.White
        End With
    End Sub

    Friend Sub TheSystem_ListViewResize()
        ResizeLastColumn()
    End Sub

    Private Sub ResizeLastColumn()
        With Form1.MyListView1
            If .Columns.Count > 1 Then
                ' --------------------------------------------------------- last column
                Dim w As Int32 = 0
                For i As Int32 = 0 To .Columns.Count - 2
                    w += .Columns(i).Width
                Next
                w = .ClientSize.Width - w - 2
                If Not ConfigurationIsValid Then
                    w = Math.Max(w, 200)
                End If
                .Columns(.Columns.Count - 1).Width = w
            End If
        End With
    End Sub



    ' ==================================================================================================
    '   LIST COMPONENTS - UPDATE VALUES
    ' ==================================================================================================

    Friend Sub TheSystem_UpdateListedSubtypes()
        Dim str As String
        With Form1.MyListView1
            If CanUseWindowsDlls Then
                .SetExtStyle_DoubleBuffering()
                .BeginUpdate()
            End If
            Dim n As Int32 = 0
            For Each m As Master In Masters
                If m.GetName <> .Items(n).SubItems(2).Text Then .Items(n).SubItems(2).Text = m.GetName
                n += 1
                For Each s As Slave In m.Slaves
                    str = s.SlaveTypeString
                    If str <> .Items(n).SubItems(2).Text Then .Items(n).SubItems(2).Text = str
                    n += 1
                    For Each p As Pin In s.Pins
                        str = Pin.PinTypeToString(p.GetPinType)
                        If str <> .Items(n).SubItems(2).Text Then .Items(n).SubItems(2).Text = str
                        n += 1
                    Next
                Next
            Next
            If CanUseWindowsDlls Then
                .EndUpdate()
            End If
        End With
    End Sub


    Friend Sub TheSystem_UpdateListedValues()
        Dim ci As Globalization.CultureInfo = New Globalization.CultureInfo("en-GB")
        Dim str As String
        With Form1.MyListView1
            If CanUseWindowsDlls Then
                ' -----------------------------------------------------------------------------
                '  DoubleBuffering must be set also here ( not only in the list-view creation )
                ' -----------------------------------------------------------------------------
                .SetExtStyle_DoubleBuffering()
                ' -----------------------------------------------------------------------------
                .BeginUpdate()
            End If
            Dim n As Int32 = 0
            For Each m As Master In Masters
                If Not m.Hid.MyDeviceDetected Then
                    .Items(n).SubItems(6).Text = "DISCONNECTED"
                    .Items(n).BackColor = Color.FromArgb(255, 150, 150)
                    .Items(n).ForeColor = Color.Black
                End If
                n += 1
                For Each s As Slave In m.Slaves
                    n += 1
                    For Each p As Pin In s.Pins
                        If p.GetPinType <> Pin.PinTypes.UNUSED Then
                            If Single.IsNaN(p.Value) Then
                                str = "Sleep"
                                'str = DecodeNan(p.Value)
                            Else
                                str = p.Value.ToString("0.0", ci)
                            End If
                            If str <> .Items(n).SubItems(5).Text Then .Items(n).SubItems(5).Text = str
                        End If
                        n += 1
                    Next
                Next
            Next
            If CanUseWindowsDlls Then
                .EndUpdate()
            End If
        End With
    End Sub

    'Private Function DecodeNan(ByVal n As Single) As String
    '    Dim ns As Single = NAN_Sleep
    '    Dim nz As Single = NAN_Zero
    '    Select Case TryCast(n, Object)
    '        Case CType(ns, Int64)
    '            Return "Sleep"
    '        Case CType(nz, Int64)
    '            Return "Nan_0"
    '        Case Else
    '            Return Ctype(n, int64).ToString
    '    End Select
    'End Function


    ' ==================================================================================================
    '   CONFIGURATIONS
    ' ==================================================================================================

    Friend ConfigDatabase() As Object

    Private Sub MarkErrorLines()
        Dim config(-1) As String
        '
        For Each m As Master In Masters

            If m.ConfigValid Then Continue For

            Dim n As Int32 = 0
            Dim sa() As String
            '
            If ConfigDatabase.Length > 0 Then
                config = TryCast(ConfigDatabase(m.ConfigId), String())
            End If

            ' TODO
            ' if already axists another master with the same name then
            'm.ConfigValid = False


            If n < config.Length Then
                sa = Split(config(n), ",")
                If sa.Length < 2 OrElse sa(0).Trim <> "Master" OrElse sa(1).Trim <> m.GetName Then
                    MarkErrorLine(n)
                End If
            Else
                MarkErrorLine(n)
            End If
            n += 1
            For Each s As Slave In m.Slaves
                If n < config.Length Then
                    sa = Split(config(n), ",")
                    If sa.Length < 2 OrElse sa(0).Trim <> "Slave" OrElse sa(1).Trim <> s.SlaveTypeString Then
                        MarkErrorLine(n)
                    End If
                Else
                    MarkErrorLine(n)
                End If
                n += 1
                For Each p As Pin In s.Pins
                    If n < config.Length Then
                        sa = Split(config(n), ",")
                        If sa.Length < 2 OrElse sa(0).Trim <> "Pin" OrElse sa(1).Trim <> Pin.PinTypeToString(p.GetPinType) Then
                            MarkErrorLine(n)
                        End If
                    Else
                        MarkErrorLine(n)
                    End If
                    n += 1
                Next
            Next
            '
            While (n < config.Length)
                AddErrorLine(config, n)
                n += 1
            End While
        Next
    End Sub

    Private Function CountConfigErrors(ByVal m As Master, ByVal Config As String()) As Int32
        Dim Errors As Int32 = 0
        Dim n As Int32 = 0
        Dim sa() As String
        '
        If n < Config.Length Then
            sa = Split(Config(n), ",")
            If sa.Length < 2 OrElse sa(0).Trim <> "Master" Then 'OrElse sa(1).Trim <> m.Name Then
                Errors += 1
            End If
        Else
            Errors += 1
        End If
        n += 1
        For Each s As Slave In m.Slaves
            If n < Config.Length Then
                sa = Split(Config(n), ",")
                If sa.Length < 2 OrElse sa(0).Trim <> "Slave" OrElse sa(1).Trim <> s.SlaveTypeString Then
                    Errors += 1
                End If
            Else
                Errors += 1
            End If
            n += 1
            For Each p As Pin In s.Pins
                If n < Config.Length Then
                    sa = Split(Config(n), ",")
                    If sa.Length < 2 OrElse sa(0).Trim <> "Pin" Then
                        Errors += 1
                    End If
                Else
                    Errors += 1
                End If
                n += 1
            Next
        Next
        '
        If Config.Length > n Then
            Errors += (Config.Length - n)
        End If
        '
        Return Errors
    End Function

    Friend Function GetConfigName(ByVal config As String()) As String
        Dim sa() As String
        sa = Split(config(0), ",")
        If sa.Length > 1 Then
            Return sa(1).Trim
        Else
            Return ""
        End If
    End Function

    Friend Function FindConfigByName(ByVal name As String) As Int32
        For i As Int32 = 0 To ConfigDatabase.Length - 1
            If name.ToUpper = GetConfigName(TryCast(ConfigDatabase(i), String())).ToUpper Then
                Return i
            End If
        Next
        Return -1
    End Function

    Friend Sub FillConfigNamesCombo(ByVal cmb As ComboBox)
        For i As Int32 = 0 To ConfigDatabase.Length - 1
            cmb.Items.Add(GetConfigName(TryCast(ConfigDatabase(i), String())))
        Next
    End Sub

    Private Sub AssignConfigurationsToMasters()
        '
        ConfigurationIsValid = True
        '
        For i As Int32 = 0 To Masters.Count - 1
            Masters(i).ConfigValid = True
            ' ----------------------------------------------------------- find configuration by name
            'm.ConfigId = -1
            'For j As Int32 = 0 To ConfigDatabase.Length - 1
            '    If Masters(j).GetName = GetConfigName(TryCast(ConfigDatabase(j), String())) Then
            '        Masters(j).ConfigId = i
            '    End If
            'Next
            Masters(i).ConfigId = FindConfigByName(Masters(i).GetName)
            ' ----------------------------------------------------------- if not found add a new configuration
            If Masters(i).ConfigId < 0 Then
                Masters(i).ConfigId = ConfigDatabase.Length
                Dim config As String() = CreateConfig_AsStringArray(Masters(i))
                AddToConfigDatabase(config)
                TheSystem_SaveConfigDatabase()
            End If
            ' ----------------------------------------------------------- test config errors
            If CountConfigErrors(Masters(i), _
                                 TryCast(ConfigDatabase(Masters(i).ConfigId), String())) > 0 Then
                Masters(i).ConfigValid = False
                ConfigurationIsValid = False
            End If
        Next
    End Sub

    Private Sub SetConfigurationParamsToDevices()
        Dim PinTypeChanged As Boolean
        Dim sa() As String
        For Each m As Master In Masters
            If m.ConfigId >= ConfigDatabase.Length Then Continue For
            Dim config As String() = TryCast(ConfigDatabase(m.ConfigId), String())
            If config Is Nothing Then Continue For
            Dim n As Int32 = 0
            If n >= config.Length Then Continue For
            sa = Split(config(n), ",")
            n += 1
            If sa.Length = 4 Then
                m.SetSpeed(CInt(Val(sa(2).Trim)))
                m.FastComm = sa(3).Trim = "True"
            End If
            For Each s As Slave In m.Slaves
                If n >= config.Length Then Continue For
                sa = Split(config(n), ",")
                n += 1
                'If sa.Length >= 2 Then
                'End If
                PinTypeChanged = False
                For Each p As Pin In s.Pins
                    If n >= config.Length Then Continue For
                    sa = Split(config(n), ",")
                    n += 1

                    Dim ConfPinType As Pin.PinTypes = Pin.StringToPinType(sa(1))

                    If ConfPinType <> p.GetPinType Then
                        s.ConfigurePin(p.PinId, ConfPinType)
                        PinTypeChanged = True
                    End If

                    If sa.Length >= 6 Then
                        p.Slot = CInt(Val(sa(2).Trim))
                        If p.Slot > 999 Then p.Slot = 999
                        p.Value_Max = CSng(Val(sa(3).Trim))
                        p.Value_Min = CSng(Val(sa(4).Trim))
                        p.ResponseSpeed = CInt(Val(sa(5).Trim))
                        If p.ResponseSpeed > 100 Then p.ResponseSpeed = 100
                        Select Case p.GetPinType
                            Case Pin.PinTypes.SERVO_8, Pin.PinTypes.SERVO_16
                                If sa.Length >= 8 Then
                                    p.MaxTime = CSng(Val(sa(6).Trim))
                                    p.MinTime = CSng(Val(sa(7).Trim))
                                End If
                            Case Pin.PinTypes.PWM_8, Pin.PinTypes.PWM_16
                                If sa.Length >= 9 Then
                                    p.MaxTime = CSng(Val(sa(6).Trim))
                                    p.MinTime = CSng(Val(sa(7).Trim))
                                    p.LogResponse = sa(8).Trim = "True"
                                End If
                            Case Pin.PinTypes.CAP_8, Pin.PinTypes.CAP_16
                                If sa.Length >= 8 Then
                                    p.MinVariation = CSng(Val(sa(6).Trim))
                                    p.ProportionalArea = CSng(Val(sa(7).Trim))
                                End If
                            Case Pin.PinTypes.USOUND_SENSOR
                                If sa.Length >= 9 Then
                                    p.MaxDist_mm = CSng(Val(sa(6).Trim))
                                    p.MinDist_mm = CSng(Val(sa(7).Trim))
                                    p.RemoveErrors = sa(8).Trim = "True"
                                End If
                            Case Pin.PinTypes.CAP_SENSOR
                                If sa.Length >= 9 Then
                                    p.MaxDist_mm = CSng(Val(sa(6).Trim))
                                    p.MinDist_mm = CSng(Val(sa(7).Trim))
                                    p.Area_cmq = CSng(Val(sa(8).Trim))
                                End If
                            Case Pin.PinTypes.COUNTER, Pin.PinTypes.COUNTER_PU, _
                                 Pin.PinTypes.FAST_COUNTER, Pin.PinTypes.FAST_COUNTER_PU, _
                                 Pin.PinTypes.PERIOD, Pin.PinTypes.PERIOD_PU
                                If sa.Length >= 9 Then
                                    p.MaxFreq = CSng(Val(sa(6).Trim))
                                    p.MinFreq = CSng(Val(sa(7).Trim))
                                    p.ConvertToFreq = sa(8).Trim = "True"
                                End If
                        End Select
                    End If
                Next

                ' -------------------------------------- re-align all the pins 
                'If PinTypeChanged Then
                m.SetupSlavePins(s.mSlaveId)
                'End If
            Next
            m.SetCommunicationIndexesToPins()
        Next
    End Sub

    Private Function CreateConfig_AsStringArray(ByVal m As Master) As String()
        Dim i As Int32
        Dim config(0) As String
        config(0) = "Master"
        config(0) &= "," & m.GetName
        config(0) &= "," & m.CommSpeed.ToString
        config(0) &= "," & m.FastComm.ToString
        For Each s As Slave In m.Slaves
            i = config.Length
            ReDim Preserve config(i)
            config(i) = "Slave"
            config(i) &= "," & s.SlaveTypeString
            For Each p As Pin In s.Pins
                i = config.Length
                ReDim Preserve config(i)
                config(i) = "Pin"
                config(i) &= "," & Pin.PinTypeToString(p.GetPinType)
                config(i) &= "," & p.Slot.ToString
                config(i) &= "," & p.Value_Max.ToString
                config(i) &= "," & p.Value_Min.ToString
                config(i) &= "," & p.ResponseSpeed.ToString
                Select Case p.GetPinType
                    Case Pin.PinTypes.SERVO_8, Pin.PinTypes.SERVO_16
                        config(i) &= "," & p.MaxTime.ToString
                        config(i) &= "," & p.MinTime.ToString
                    Case Pin.PinTypes.PWM_8, Pin.PinTypes.PWM_16
                        config(i) &= "," & p.MaxTime.ToString
                        config(i) &= "," & p.MinTime.ToString
                        config(i) &= "," & p.LogResponse.ToString
                    Case Pin.PinTypes.CAP_8, Pin.PinTypes.CAP_16
                        config(i) &= "," & p.MinVariation.ToString
                        config(i) &= "," & p.ProportionalArea.ToString
                    Case Pin.PinTypes.USOUND_SENSOR
                        config(i) &= "," & p.MaxDist_mm.ToString
                        config(i) &= "," & p.MinDist_mm.ToString
                        config(i) &= "," & p.RemoveErrors.ToString
                    Case Pin.PinTypes.CAP_SENSOR
                        config(i) &= "," & p.MaxDist_mm.ToString
                        config(i) &= "," & p.MinDist_mm.ToString
                        config(i) &= "," & p.Area_cmq.ToString
                    Case Pin.PinTypes.COUNTER, Pin.PinTypes.COUNTER_PU, _
                         Pin.PinTypes.FAST_COUNTER, Pin.PinTypes.FAST_COUNTER_PU, _
                         Pin.PinTypes.PERIOD, Pin.PinTypes.PERIOD_PU
                        config(i) &= "," & p.MaxFreq.ToString
                        config(i) &= "," & p.MinFreq.ToString
                        config(i) &= "," & p.ConvertToFreq.ToString
                End Select
            Next
        Next
        Return config
    End Function

    Friend Sub TheSystem_ValidateConfiguration()
        For Each m As Master In Masters
            m.ConfigValid = True
        Next
        ConfigurationIsValid = True
        TheSystem_ListComponents()
        Form1.ListView_SetAllLineColors()
        Form1.Menu_File_Validate.Enabled = Not ConfigurationIsValid
        Form1.ToolStripButton_Validate.Enabled = Not ConfigurationIsValid
        Form1.EnableCalibrateButton(TheSystem_CalibrationNeeded)
    End Sub

    Friend Sub AddToConfigDatabase(ByVal config As String())
        If config.Length > 0 Then
            Dim count As Int32 = ConfigDatabase.Length
            ReDim Preserve ConfigDatabase(count)
            ConfigDatabase(count) = config
        End If
    End Sub

    Friend Sub TheSystem_UpdateAndSaveConfigDatabase()
        TheSystem_UpdateConfigDatabase()
        TheSystem_SaveConfigDatabase()
    End Sub

    Friend Sub TheSystem_UpdateConfigDatabase()
        If ConfigurationIsValid Then
            For Each m As Master In Masters
                If m.ConfigValid Then
                    ConfigDatabase(m.ConfigId) = CreateConfig_AsStringArray(m)
                End If
            Next
        End If
    End Sub

    Friend Sub TheSystem_SaveConfigDatabase()
        If ConfigurationIsValid Then
            Save_ConfigDatabase()
        Else
            'MsgBox("Invalid configuration - ""Config Database"" not saved.", MsgBoxStyle.Information)
        End If
    End Sub

End Module
